# Run daily by cron to download and apply modifications to geonames database.
# Output is sent (by cron) to '##PATH TO YOUR LOG FILE##/geonamesDaily.log'.

# Geonames download server changed from download.geonames.org to 
# api.geonames.org, Feb 2011.

# Job split into 2 steps, March 2012:
# step1: download the update files from geonames server
# step2: apply them
# Reason for split is that download is the step that sometimes fails (and we
# want to know asap) whereas step2 can be slow if lots of updates, so do it
# late in the evening.

# Changed back to download.geonames.org server because api.geonames.org server
# is just too unreliable. Nov 2012.
 
# Amended to include timestamp on log entries, Nov 2012.

# THIS IS STEP 1.

# == Variables and functions ==================================================

MYSQLDIR=##LOCATION OF YOUR MYSQL DATABASE##
DAILYDIR=$MYSQLDIR/geonamesDailyFiles
ERRDIR=$MYSQLDIR/preservedUpdates
YESTERDAY=`date -d 'yesterday' +%Y-%m-%d`
DAYBEFOREYESTERDAY=`date -d '2 days ago' +%Y-%m-%d`
DOWNLOADERROR=""

# function called when any of the expected download files were not found
handleDownloadError()
# arg1 is string containing names of missing files
{
  mail -s "Geonames download failed" k.byrne@ed.ac.uk,##INSERT YOUR EMAIL ADDRESS (ES) HERE## <<EOF
One or more of the expected daily update files from geonames was not successfully downloaded.
Missing files:
$1

This may be an error with the 'wget' process, or a problem with the geonames 
server (at http://download.geonames.org/export/dump). Check the destination
##PATH TO YOUR DIRECTORY##/geonamesDailyFiles directory, the
##PATH TO YOUR DIRECTORY##/wget.log file and the geonames server site.

If any files *were* downloaded they have been kept in case they need to be 
reapplied later: ##PATH TO YOUR DIRECTORY##/preservedUpdates.
EOF
  # add note to log file too
  echo `date`": Error occurred when downloading files for $YESTERDAY."
  # preserve update files if errors occurred (in case *something* is present)
  cp -p $DAILYDIR/* $ERRDIR
}

# -----------------------------------------------------------------------------

# == Collect update files from geonames server ================================
 
# remove day before yesterday's files
rm $DAILYDIR/*$DAYBEFOREYESTERDAY.txt* $DAILYDIR/hierarchy.*

# download daily update files from geonames server to DAILYDIR on scargill
# ('nv' option means only minimal logging; 'o' overwrites logfile; 'a' appends)
wget -nv -P $DAILYDIR -o $MYSQLDIR/wget.log http://download.geonames.org/export/dump/modifications-$YESTERDAY.txt
wget -nv -P $DAILYDIR -a $MYSQLDIR/wget.log http://download.geonames.org/export/dump/deletes-$YESTERDAY.txt
wget -nv -P $DAILYDIR -a $MYSQLDIR/wget.log http://download.geonames.org/export/dump/alternateNamesModifications-$YESTERDAY.txt
wget -nv -P $DAILYDIR -a $MYSQLDIR/wget.log http://download.geonames.org/export/dump/alternateNamesDeletes-$YESTERDAY.txt
wget -nv -P $DAILYDIR -a $MYSQLDIR/wget.log http://download.geonames.org/export/dump/hierarchy.zip

# wait for these downloads to finish - server can be slow.
wait `ps -ef|awk '/wget/{print $2}'` > /dev/null 2>&1

###########################################################
# BELT AND BRACES FIX - wait NOT WORKING PROPERLY? So do wait then pause for
# another couple of minutes!
sleep 120

# -----------------------------------------------------------------------------

# == For each file: check download worked ok ==

# 1) -- geoname updates -------------------------------------------------------

if [ -e "$DAILYDIR/modifications-$YESTERDAY.txt" ]
then
  # file was successfully downloaded
  :
else
  # download problem
  DOWNLOADERROR="$DOWNLOADERROR modifications-$YESTERDAY.txt;"
fi

# 2) -- geoname deletes -------------------------------------------------------

if [ -e "$DAILYDIR/deletes-$YESTERDAY.txt" ]
then
  # file was successfully downloaded
  :
else
  # download problem
  DOWNLOADERROR="$DOWNLOADERROR deletes-$YESTERDAY.txt;"
fi

# 3) -- alternatename updates -------------------------------------------------

if [ -e "$DAILYDIR/alternateNamesModifications-$YESTERDAY.txt" ]
then
  # file was successfully downloaded
  :
else
  # download problem
  DOWNLOADERROR="$DOWNLOADERROR alternateNamesModifications-$YESTERDAY.txt;"
fi

# 4) -- alternatename deletes -------------------------------------------------

if [ -e "$DAILYDIR/alternateNamesDeletes-$YESTERDAY.txt" ]
then
  # file was successfully downloaded
  :
else
  # download problem
  DOWNLOADERROR="$DOWNLOADERROR alternateNamesDeletes-$YESTERDAY.txt;"
fi

# --  Recreate hierarchy table each day -----------------------------------------

if [ -e "$DAILYDIR/hierarchy.zip" ]
then
  # file was successfully downloaded
  :
else
  # download problem
  DOWNLOADERROR="$DOWNLOADERROR hierarchy.zip"
fi

# -----------------------------------------------------------------------------

# Send single email if any download errors occurred.
if [[ -n $DOWNLOADERROR ]] 
then
  # variable string contains names of files that didn't download
  handleDownloadError "$DOWNLOADERROR"
else
  # note in log that downloads succeeded
  echo `date`": Files for $YESTERDAY downloaded ok."
fi

# -----------------------------------------------------------------------------
